@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                                   Body                                   */
 /*                                                                          */
-/*                              $Revision: 1.16 $                           */
+/*                              $Revision: 1.25 $                           */
 /*                                                                          */
-/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -39,13 +39,31 @@
 #include <sys/stat.h>
 #include <stdio.h>
 
-#if defined (__EMX__) || defined (WINNT)
+#if defined (__EMX__)
 #include <stdlib.h>
 int max_path_len = _MAX_PATH;
 #else
+#ifdef VMS
+#include <unixlib.h>
+/* #include <limits.h> */
+int max_path_len = 255; /* PATH_MAX */
+#else
+#ifdef __vxworks
+#include <limits.h>
+int max_path_len = PATH_MAX;
+#else
 #include <sys/param.h>
+
+/* If we can't find MAXPATHLEN there, try in unistd.h.  That file might
+   not exist, but we'll blow up anyway if we don't try something.  */
+#ifndef MAXPATHLEN
+#include <unistd.h>
+#endif
+
 int max_path_len = MAXPATHLEN;
 #endif
+#endif
+#endif
 
 int feof__ (FILE *stream)
 {
@@ -81,10 +99,16 @@ is_regular_file_fd (fd)
 #define SEEK_END 2  /* Set file pointer to the size of the file plus offset */
 #endif
 
+/* if L_tmpnam is not set, use a large number that should be safe */
+#ifndef L_tmpnam
+#define L_tmpnam 256
+#endif
+
 int    c_constant_eof      = EOF;
 int    c_constant_iofbf    = _IOFBF;
 int    c_constant_iolbf    = _IOLBF;
 int    c_constant_ionbf    = _IONBF;
+int    c_constant_l_tmpnam = L_tmpnam;
 int    c_constant_seek_cur = SEEK_CUR;
 int    c_constant_seek_end = SEEK_END;
 int    c_constant_seek_set = SEEK_SET;
@@ -98,20 +122,30 @@ char *
 full_name (char *nam, char *buffer)
 {
    char *p;
-#if defined (__EMX__) || defined (WINNT)
+#if defined (__EMX__)
    _fullpath (buffer, nam, max_path_len);
    for (p = buffer; *p; p++) if (*p == '/') *p = '\\';
 
-#else
-#if defined (MSDOS)
+#elif defined (MSDOS)
    _fixpath (nam, buffer);
 
-#else
-#if defined (sgi) || defined (sun) || defined (linux)
+#elif defined (sgi) || defined (sun) || defined (linux)
    /* Use realpath function which resolves links and references to .. and ..
       on those Unix systems that support it. */
    realpath (nam, buffer);
 
+#elif defined (VMS)
+   /* This only handles pure filenames and absolute paths and ignores anything
+      with a relative path since VMS paths cant be simply concatenated
+      together, but to do it right requires RMS services and the header files
+      arent in a format that gcc can use.  Fix later ??? */
+   if (strchr (nam, '[') || strchr (nam, ':')) {
+      strcpy (buffer, nam);
+   }
+   else {
+      buffer = getcwd (buffer, max_path_len);
+      strcat (buffer, nam);
+   }
 #else
    extern char *getcwd();
    if (nam[0] != '/') {
@@ -123,6 +157,4 @@ full_name (char *nam, char *buffer)
       strcpy (buffer, nam);
    }
 #endif
-#endif
-#endif
 }
