@@ -6,7 +6,7 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.3 $                              */
+/*                            $Revision: 1.4 $                              */
 /*                                                                          */
 /*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
@@ -61,7 +61,7 @@ extern struct Elmt_Item *Elmts_Ptr;
 /* Element List Access Functions:  */
 
 INLINE Node_Id
-Id_Of (Elmt)
+Node (Elmt)
      Elmt_Id Elmt;
 {
   return Elmts_Ptr [Elmt].node;
