@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.24 $                             */
+/*                            $Revision: 1.32 $                             */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,13 +18,21 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
-/*  Definitions to access the front-end's eval functions. */
-#define Expr_Value   sem_eval__expr_value
-#define Expr_Value_S sem_eval__expr_value_s
+/*  Definitions to access front-end functions used in gigi */
+#define Entity_Is_In_Main_Unit lib__entity_is_in_main_unit
+extern Boolean Entity_Is_In_Main_Unit PROTO((Entity_Id));
+#define Expr_Value		sem_eval__expr_value
+extern Uint Expr_Value		PROTO((Node_Id));
+#define Expr_Value_S		sem_eval__expr_value_s
+extern Node_Id Expr_Value_S	PROTO((Node_Id));
 
 /* Given GNAT_ENTITY, a GNAT defining identifier node, which denotes some Ada
    entity, this routine returns the equivalent GCC tree for that entity
@@ -33,6 +41,7 @@
 
    If GNAT_ENTITY is a variable or a constant declaration, GNU_EXPR gives its
    initial value (in GCC tree form). This is optional for variables.
+   For renamed entities, GNU_EXPR gives the object being renamed.
 
    DEFINITION is nonzero if this call is intended for a definition.  This is
    used for separate compilation where it necessary to know whether an
@@ -54,16 +63,28 @@ extern void elaborate_entity	PROTO((Entity_Id));
 /* Make a dummy type corresponding to GNAT_TYPE.  */
 extern tree make_dummy_type	PROTO((Entity_Id));
 
-/* Given GNAT_ENTITY of kind E_Function or E_Procedure, modify the declaration
-   to include the attribute, if it is valid for this configuration */
-extern tree maybe_machine_attribute PROTO((Entity_Id));
+/* Get the unpadded version of a GNAT type.  */
+extern tree get_unpadded_type	PROTO((Entity_Id));
 
 /* Called when we need to protect a variable object using a save_expr.  */
-extern tree maybe_variable	PROTO((tree));
+extern tree maybe_variable	PROTO((tree, Node_Id));
+
+/* Ensure that TYPE has SIZE and ALIGN.  Make and return a new padded type
+   if needed.  We have already verified that SIZE and TYPE are large enough.
+
+   GNAT_ENTITY and NAME_TRAILER are used to name the resulting record and
+   to issue a warning. 
+
+   IN_PLACE is nonzero if we can modify the type in place; otherwise
+   we must make a new one to make any changes to the size and alignment.  */
+extern tree maybe_pad_type	PROTO((tree, tree, int, Entity_Id,
+				       char *, int));
 
 /* Given a GNU tree and a GNAT list of choices, generate an expression to test
-   the value passed against the list of choices.  */
-extern tree choices_to_gnu	PROTO((tree, Node_Id));
+   the value passed against the list of choices.  If STRING is non-zero,
+   concatenate a discription of the choice to it.  Set STRING[0] to zero
+   if we can't.  */
+extern tree choices_to_gnu	PROTO((tree, Node_Id, char *));
 
 /* Given a type T, a FIELD_DECL F, and a replacement value R,
    return a new type with all size expressions that contain F
@@ -71,7 +92,10 @@ extern tree choices_to_gnu	PROTO((tree, Node_Id));
    substitute_in_type except that it knows about TYPE_INDEX_TYPE.  */
 extern tree gnat_substitute_in_type PROTO((tree, tree, tree));
 
+/* Return the "RM size" of GNU_TYPE.  This is the actual number of bits
+   needed to represent the object.  */
+extern tree rm_size		PROTO((tree));
+
 /* Return a name for GNAT_ENTITY concatenated with two underscores and
    STRING.  */
-
-extern char *create_concat_name PROTO((Entity_Id, char *));
+extern tree create_concat_name PROTO((Entity_Id, char *));
