@@ -6,7 +6,7 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                             $Revision: 1.94 $                            */
+/*                             $Revision: 1.123 $                           */
 /*                                                                          */
 /*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
@@ -38,6 +38,9 @@
 #include "tree.h"
 #include "rtl.h"
 #include "expr.h"
+#include "insn-flags.h"
+#include "insn-config.h"
+#include "recog.h"
 #include "a-ada.h"
 #include "a-types.h"
 #include "a-atree.h"
@@ -56,14 +59,6 @@
 #include "a-rtree.h"
 #include "flags.h"
 
-/* This is needed for Linux since "setjmp" is a macro defined to be a function
-   of a different name.  This file MUST NOT be included for other systems
-   because of the potential of conflicts with the way we kludge uses of
-   setjmp at the moment.  */
-#ifdef linux
-#include <setjmp.h>
-#endif
-
 extern char *xmalloc ();
 extern char *main_input_filename;
 
@@ -123,17 +118,16 @@ int
 yyparse ()
 {
   /* Make up what Gigi uses as a jmpbuf.  */
-  size_t jmpbuf[100];
+  size_t jmpbuf[10];
+
+  /* call the target specific initializations */
+  __gnat_initialize();
 
   /* Call the front-end elaboration procedures */
-  ada__bind ();
+  adainit ();
 
   /* Set up to catch unhandled exceptions.  */
-#if defined (sparc) && defined (__svr4__)
-  if (sigsetjmp (jmpbuf))
-#else
-  if (setjmp (jmpbuf))
-#endif
+  if (__builtin_setjmp (jmpbuf))
     abort ();
 
   system__task_specific_data__set_jmpbuf_address (jmpbuf);
@@ -162,36 +156,26 @@ init_gnat_args ()
   gnat_argc = 2;
 }
 
-/* Decode all the language specific options that cannot be decoded by GCC. The
-   option decoding phase of GCC calls this routine on the flags that it cannot
-   decode. This routine returns 1 if it is successful, otherwise it
-   returns 0. */
+/* Decode all the language specific options that cannot be decoded by GCC.
+   The option decoding phase of GCC calls this routine on the flags that
+   it cannot decode. This routine returns 1 if it is successful, otherwise
+   it returns 0. */
 
 int
 lang_decode_option (p)
      char *p;
 {
   if (!gnat_argc) init_gnat_args ();
-#if defined (WINNT) || defined (__EMX__)
-  if (strnicmp (p, "-gnat", 5))
-#else
-  if (strncasecmp (p, "-gnat", 5))
-#endif
-    if (!strncmp (p, "-I", 2))
-       {
-       /* pass the -I switches as-is */
-       gnat_argv[gnat_argc] = p;
-       gnat_argc ++;
-       return 1;
-       }
-  else
-    {
-       /* we assume for the moment that all other front end options
-          are passed with "gnat" prefix */
-       return 0;
-       }
 
-  else
+  if (!strncmp (p, "-I", 2))
+    {
+      /* pass the -I switches as-is */
+      gnat_argv[gnat_argc] = p;
+      gnat_argc ++;
+      return 1;
+    }
+
+  if (!strncmp (p, "-gnat", 5))
     {
       /* recopy the switches without the 'gnat' prefix */
 
@@ -201,6 +185,8 @@ lang_decode_option (p)
       gnat_argc ++;
       return 1;
     }
+
+  return 0;
 }
 
 /* Perform all the initialization steps that are language-specific.  */
@@ -275,20 +261,40 @@ print_lang_type (file, node, indent)
      tree node;
      int indent;
 {
-  if (TREE_CODE (node) == FUNCTION_TYPE && TYPE_CI_CO_LIST (node))
-    print_node (file, "ci_co_list", TYPE_CI_CO_LIST (node), indent + 4);
-  else if (TREE_CODE (node) == INTEGER_TYPE && TYPE_MODULAR_P (node)
-	   && TYPE_MODULUS (node))
-    print_node (file, "modulus", TYPE_MODULUS (node), indent + 4);
-  else if (TREE_CODE (node) == INTEGER_TYPE && TYPE_INDEX_TYPE (node))
-    print_node (file, "index type", TYPE_INDEX_TYPE (node), indent + 4);
-  else if (TREE_CODE (node) == RECORD_TYPE && TYPE_FAT_POINTER_P (node)
-	   && TYPE_UNCONSTRAINED_ARRAY (node))
-    print_node (file, "unconstrained array",
-		TYPE_UNCONSTRAINED_ARRAY (node), indent + 4);
-  else if (TREE_CODE (node) == RECORD_TYPE && TYPE_PARENT_SUBTYPE (node))
-    print_node (file, "parent subtype",
-		TYPE_PARENT_SUBTYPE (node), indent + 4);
+  switch (TREE_CODE (node))
+    {
+    case FUNCTION_TYPE:
+      print_node (file, "ci_co_list", TYPE_CI_CO_LIST (node), indent + 4);
+      break;
+
+    case ENUMERAL_TYPE:
+      print_node (file, "RM size", TYPE_RM_SIZE_ENUM (node), indent + 4);
+      break;
+
+    case INTEGER_TYPE:
+      if (TYPE_MODULAR_P (node))
+	print_node (file, "modulus", TYPE_MODULUS (node), indent + 4);
+      else if (TYPE_HAS_ACTUAL_BOUNDS_P (node))
+	print_node (file,"actual bounds", TYPE_ACTUAL_BOUNDS (node),
+		    indent + 4);
+      else
+	print_node (file, "index type", TYPE_INDEX_TYPE (node), indent + 4);
+
+      print_node (file, "RM size", TYPE_RM_SIZE_INT (node), indent + 4);
+      break;
+
+    case ARRAY_TYPE:
+      print_node (file,"actual bounds", TYPE_ACTUAL_BOUNDS (node), indent + 4);
+      break;
+
+    case RECORD_TYPE:
+      if (TYPE_IS_FAT_POINTER_P (node) || TYPE_CONTAINS_TEMPLATE_P (node))
+	print_node (file, "unconstrained array",
+		    TYPE_UNCONSTRAINED_ARRAY (node), indent + 4);
+      else
+	print_node (file, "Ada size", TYPE_ADA_SIZE (node), indent + 4);
+      break;
+    }
 }
 
 void
@@ -338,13 +344,23 @@ gnat_expand_expr (exp, target, tmode, modifier)
       break;
 
     case UNCHECKED_CONVERT_EXPR:
+      /* If we're converting between an aggregate and non-aggregate type
+	 and we have a MEM TARGET, we can't use it, since MEM_IN_STRUCT_P
+	 would be set incorrect.  */
+      if (target != 0 && GET_CODE (target) == MEM
+	  && (MEM_IN_STRUCT_P (target)
+	      != AGGREGATE_TYPE_P (TREE_TYPE (TREE_OPERAND (exp, 0)))))
+	target = 0;
+
       /* If the input and output are both the same mode (usually BLKmode),
 	 just return the expanded input since we want just the bits.  But
 	 we can't do this if the output is more strictly aligned than
 	 the input.  */
       if (TYPE_MODE (type) == TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0)))
-	  && (TYPE_ALIGN (type)
-	      <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))))
+	  && ((TYPE_ALIGN (type)
+	       <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0))))
+	      || (TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))
+		  >= BIGGEST_ALIGNMENT)))
 	new = TREE_OPERAND (exp, 0);
 
       /* If either mode is BLKmode, memory will be involved, so do this
@@ -355,6 +371,8 @@ gnat_expand_expr (exp, target, tmode, modifier)
 		|| TYPE_MODE (TREE_TYPE (TREE_OPERAND (exp, 0))) == BLKmode)
 	       && ((TYPE_ALIGN (type)
 		    <= TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0))))
+		   || (TYPE_ALIGN (TREE_TYPE (TREE_OPERAND (exp, 0)))
+		       >= BIGGEST_ALIGNMENT)
 		   || TYPE_ALIGN_OK_P (type)
 		   || TYPE_ALIGN_OK_P (TREE_TYPE (TREE_OPERAND (exp, 0)))))
 	{
@@ -389,8 +407,10 @@ gnat_expand_expr (exp, target, tmode, modifier)
 
 	  in_type = TREE_TYPE (TREE_OPERAND (exp, 0));
 	  union_type = make_node (UNION_TYPE);
-	  in_field = create_field_decl ("in", in_type, union_type, 0, -1, 0);
-	  out_field = create_field_decl ("out", type, union_type, 0, -1, 0);
+	  in_field = create_field_decl (get_identifier ("in"),
+					in_type, union_type, 0, 0, 0);
+	  out_field = create_field_decl (get_identifier ("out"),
+					 type, union_type, 0, 0, 0);
 
 	  TYPE_FIELDS (union_type) = chainon (in_field, out_field);
 	  layout_type (union_type);
@@ -453,6 +473,10 @@ gnat_expand_expr (exp, target, tmode, modifier)
       emit_insn (gen_rtx (USE, VOIDmode, result));
       return target;
 
+    case GNAT_NOP_EXPR:
+      return expand_expr (build1 (NOP_EXPR, type, TREE_OPERAND (exp, 0)),
+			  target, tmode, modifier);
+
     case UNCONSTRAINED_ARRAY_REF:
       /* If we are evaluating just for side-effects, just evaluate our
 	 operand.  Otherwise, abort since this code should never appear
@@ -485,6 +509,38 @@ make_transform_expr (gnat_node, gnu_type)
   return gnu_result;
 }
 
+/* Update the setjmp buffer BUF with the current stack pointer.  We assume
+   here that a __builtin_setjmp was done to BUF.  */
+
+void
+update_setjmp_buf (buf)
+     tree buf;
+{
+  enum machine_mode sa_mode = Pmode;
+  rtx stack_save;
+
+#ifdef HAVE_save_stack_nonlocal
+  if (HAVE_save_stack_nonlocal)
+    sa_mode = insn_operand_mode[(int) CODE_FOR_save_stack_nonlocal][0];
+#endif
+
+  stack_save
+    = gen_rtx (MEM, sa_mode,
+	       memory_address
+	       (sa_mode,
+		plus_constant (expand_expr (build_unary_op (ADDR_EXPR,
+							    NULL_TREE, buf),
+					    NULL_RTX, VOIDmode, 0),
+			       2 * GET_MODE_SIZE (Pmode))));
+
+#ifdef HAVE_setjmp
+  if (HAVE_setjmp)
+    emit_insn (gen_setjmp ());
+#endif
+
+  emit_stack_save (SAVE_NONLOCAL, &stack_save, NULL_RTX);
+}
+
 /* Record the current code position in GNAT_NODE.  */
 
 void
@@ -509,15 +565,16 @@ insert_code_for (gnat_node)
   if (global_bindings_p ())
     {
       push_pending_elaborations ();
+      gnat_to_code (gnat_node);
       insert_elaboration_list (get_gnu_tree (gnat_node));
       pop_pending_elaborations ();
-      gnat_to_code (gnat_node);
     }
   else
     {
       rtx insns;
 
       start_sequence ();
+      mark_all_temps_used ();
       gnat_to_code (gnat_node);
       insns = get_insns ();
       end_sequence ();
@@ -591,18 +648,37 @@ get_type_alignment (gnat_type)
   return TYPE_ALIGN (gnat_to_gnu_type (gnat_type)) / BITS_PER_UNIT;
 }
 
-/* Utility Routines needed by the Tree Translator: */
-
-/* GNU_TYPE is the type of a subprogram parameter.  Determine from the type if
-   it should be passed by reference.  */
+/* GNU_TYPE is a type. Determine if it should be passed by reference by
+   default.  */
 
 int
-pass_by_ref (gnu_type)
+default_pass_by_ref (gnu_type)
      tree gnu_type;
 {
-  /* We pass only BLKmode, unconstrained objects, and those required
-     by the language by reference.  */
+  /* We pass aggregates by reference if they are sufficiently large.
+     The choice of constant here is somewhat arbitrary.  */
+  return (AGGREGATE_TYPE_P (gnu_type)
+	  && (TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST
+	      || TREE_INT_CST_HIGH (TYPE_SIZE (gnu_type)) != 0
+	      || (TREE_INT_CST_LOW (TYPE_SIZE (gnu_type))
+		  > (HOST_WIDE_INT) 8 * BITS_PER_WORD)
+	      || TREE_OVERFLOW (TYPE_SIZE (gnu_type))));
+}
+
+/* GNU_TYPE is the type of a subprogram parameter.  Determine from the type if
+   it should be passed by reference. */
+
+int
+must_pass_by_ref (gnu_type)
+     tree gnu_type;
+{
+  /* We pass only unconstrained objects, those required by the language
+     to be passed by reference, and objects of variable size.  The latter
+     is more efficient, avoids problems with variable size temporaries,
+     and does not produce compatibility problems with C, since C does
+     not have such objects.  */
   return (TREE_CODE (gnu_type) == UNCONSTRAINED_ARRAY_TYPE
-	  || TYPE_MODE (gnu_type) == BLKmode
-	  || (AGGREGATE_TYPE_P (gnu_type) && TYPE_BY_REFERENCE_P (gnu_type)));
+	  || (AGGREGATE_TYPE_P (gnu_type) && TYPE_BY_REFERENCE_P (gnu_type))
+	  || (TYPE_SIZE (gnu_type) != 0
+	      && TREE_CODE (TYPE_SIZE (gnu_type)) != INTEGER_CST));
 }
