@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.15 $                             */
+/*                            $Revision: 1.24 $                             */
 /*                                                                          */
-/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -36,14 +36,10 @@
    GNAT Run Time Library */
 
 #include "config.h"
-
-#ifdef WINNT
 #include <fcntl.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <errno.h>
-#endif
-
 #include <stdio.h>
 
 /*
@@ -177,7 +173,7 @@
 #endif
 
 #if defined (linux) || defined (sun) || defined (sgi) || defined (__EMX__) \
-    || defined (__osf__)
+    || defined (__osf__) || defined (WINNT)
 #include <termios.h>
 #include <fcntl.h>
 #endif
@@ -211,7 +207,7 @@ getc_immediate_common
   (FILE *stream, int *ch, int *end_of_file, int *avail, int waiting)
 {
 #if defined (linux) || defined (sun) || defined (sgi) || defined (__EMX__) \
-    || defined (__osf__)
+    || defined (__osf__) || defined (WINNT)
   char c;
   int nread;
   int flags;
@@ -253,7 +249,11 @@ getc_immediate_common
           fcntl (fd, F_SETFL, O_NONBLOCK | flags);
 #endif
 
-          nread = fread (&c, 1, 1, stream);
+          /* Read() is used here instead of fread(), because fread() doesn't
+             work on Solaris5 and Sunos4 in this situation.  Maybe because we
+             are mixing calls that use file descriptors and streams. */
+
+          nread = read (fd, &c, 1);
           if (nread > 0)
             {
               /* On Unix terminals, Ctrl-D (EOT) is an End of File. */
@@ -270,20 +270,6 @@ getc_immediate_common
                   *end_of_file = 0;
                   good_one = 1;
                 }
-              /* Stuff these characters back into the buffer, reset the
-                 terminal modes back to normal and let somebody else grab it
-                 I thought this might be necessary for Ctrl-C and Ctrl-Z
-                 processing by the shell to work, but evidently not.
-                 Keep it around just in case. */
-              else if (c != c)
-                {
-                  good_one = 0;
-                  ungetc (c, stream);
-#if defined(linux)
-                  fcntl (fd, F_SETFL, flags);
-#endif
-                  tcsetattr (fd, TCSANOW, &otermios_rec);
-                }
             }
           else if (!waiting)
             {
@@ -327,31 +313,29 @@ getc_immediate_common
    Ada programs */
 
 #ifdef WINNT
+#include <windows.h>
 
-#include <windows.h>                /* required for all Windows applications */
-
-HANDLE hInstance_global;            /* current instance                      */
-HANDLE hPrevInstance_global;        /* previous instance            */
-LPSTR lpCmdLine_global;             /* command line                 */
-int nCmdShow_global;                /* show-window type (open/icon) */
-
-extern int __argc;
-extern char **__argv;
-extern char **environ;
-
-int PASCAL WinMain(hInstance, hPrevInstance, lpCmdLine, nCmdShow)
-HANDLE hInstance;                   /* current instance             */
-HANDLE hPrevInstance;               /* previous instance            */
-LPSTR lpCmdLine;                    /* command line                 */
-int nCmdShow;                       /* show-window type (open/icon) */
-{
-
-  hInstance_global = hInstance;
-  hPrevInstance_global = hPrevInstance;
-  lpCmdLine_global = lpCmdLine;
-  nCmdShow_global = nCmdShow;
-
-  return main (__argc, __argv);  /* Should return value from PostQuitMessage */
-}
+/* Provide functions to echo the values passed to WinMain (windows bindings
+   will want to import these).  We use the same names as the routines used
+   by AdaMagic for compatibility.  */
+char *rts_get_hInstance     (void) { return (GetModuleHandleA (0)); }
+char *rts_get_hPrevInstance (void) { return (0); }
+char *rts_get_lpCommandLine (void) { return (GetCommandLineA ()); }
+int   rts_get_nShowCmd      (void) { return (1); }
 
 #endif /* WINNT */
+
+#ifdef VMS
+/* This gets around a problem with using the old threads library on VMS 7.0 */
+#include <time.h>
+long
+get_gmtoff ()
+{
+  time_t t;
+  struct tm *ts;
+
+  t = time ((time_t) 0);
+  ts = localtime (&t);
+  return ts->tm_gmtoff;
+}
+#endif
