@@ -6,7 +6,7 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.3 $                              */
+/*                            $Revision: 1.4 $                              */
 /*                                                                          */
 /*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
@@ -35,6 +35,10 @@
 /* This file contains system dependent symbols that are referenced in the
    GNAT Run Time Library */
 
+#ifdef __vxworks
+#include <types/vxTypesOld.h>
+#endif
+
 #include <time.h>
 
 /* Return the value of the "time" C library function.  We always return
