@@ -6,7 +6,7 @@
 /*                                                                          */
 /*                              C Header File                               */
 /*                                                                          */
-/*                            $Revision: 1.90 $                             */
+/*                            $Revision: 1.104 $                             */
 /*                                                                          */
 /*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
 /*                                                                          */
@@ -70,8 +70,16 @@ extern int force_global;
 
 /* Flags added to GCC type nodes.  */
 
-/* Nonzero if this type is a record corresponding to a fat pointer.  */
-#define TYPE_FAT_POINTER_P(NODE) TYPE_LANG_FLAG_0 (NODE)
+/* For RECORD_TYPE, nonzero if this is a record being used
+   as a fat pointer.  */
+#define TYPE_IS_FAT_POINTER_P(NODE) TYPE_LANG_FLAG_0 (NODE)
+
+#define TYPE_FAT_POINTER_P(NODE)  \
+  (TREE_CODE (NODE) == RECORD_TYPE && TYPE_IS_FAT_POINTER_P (NODE))
+
+/* For integral types, nonzero if this is a packed array type.  Such
+   types should not be extended to a larger size.  */
+#define TYPE_PACKED_ARRAY_TYPE_P(NODE) TYPE_LANG_FLAG_0 (NODE)
 
 /* For INTEGER_TYPE, nonzero if this is a modular type with a modulus that
    is not equal to two to the power of its mode's size.  */
@@ -85,8 +93,8 @@ extern int force_global;
    unconstrained array or record.  */
 #define TYPE_RETURNS_UNCONSTRAINED_P(NODE) TYPE_LANG_FLAG_1 (NODE)
 
-/* For RECORD_TYPE, nonzero if this type has a record rep clause.  */
-#define TYPE_HAS_REP_CLAUSE_P(NODE)  TYPE_LANG_FLAG_1 (NODE)
+/* For RECORD_TYPE, nonzero if this denotes a left-justified modular type.  */
+#define TYPE_LEFT_JUSTIFIED_MODULAR_P(NODE) TYPE_LANG_FLAG_1 (NODE)
 
 /* Nonzero in an arithmetic subtype if this is a subtype not known to the
    front-end.  */
@@ -95,32 +103,50 @@ extern int force_global;
 /* Nonzero for composite types if this is a by-reference type.  */
 #define TYPE_BY_REFERENCE_P(NODE) TYPE_LANG_FLAG_2 (NODE)
 
-/* Nonzero if this is the type for an object whose type includes
-   its template in addition to its value.  */
+/* For RECORD_TYPE, nonzero if this is the type for an object whose type
+   includes its template in addition to its value.  */
 #define TYPE_CONTAINS_TEMPLATE_P(NODE) TYPE_LANG_FLAG_3 (NODE)
 
+/* True if NODE is a thin pointer.  */
+#define TYPE_THIN_POINTER_P(NODE)			\
+  (TREE_CODE (NODE) == POINTER_TYPE			\
+   && TREE_CODE (TREE_TYPE (NODE)) == RECORD_TYPE	\
+   && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (NODE)))
+
+/* True if TYPE is either a fat or thing pointer to an unconstrained
+   array.  */
+#define TYPE_FAT_OR_THIN_POINTER_P(NODE) \
+  (TYPE_FAT_POINTER_P (NODE) || TYPE_THIN_POINTER_P (NODE))
+
+/* For INTEGER_TYPEs, nonzero if the type has a biased representation.  */
+#define TYPE_BIASED_REPRESENTATION_P(NODE) TYPE_LANG_FLAG_4 (NODE)
+
 /* For ARRAY_TYPEs, nonzero if the array type has Convention_Fortran.  */
 #define TYPE_CONVENTION_FORTRAN_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
 /* For FUNCTION_TYPEs, nonzero if the function returns by reference.  */
 #define TYPE_RETURNS_BY_REF_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
-/* For VOID_TYPE and RECORD_TYPE, nonzero if this is a dummy type,
-   make to correspond to a private or incomplete type.  */
+/* For VOID_TYPE, UNION_TYPE, and RECORD_TYPE, nonzero if this is a dummy
+   type, made to correspond to a private or incomplete type.  */
 #define TYPE_DUMMY_P(NODE) TYPE_LANG_FLAG_4 (NODE)
 
 /* True if TYPE is such a dummy type.  */
 #define TYPE_IS_DUMMY_P(NODE) \
-  ((TREE_CODE (NODE) == VOID_TYPE || TREE_CODE (NODE) == RECORD_TYPE) \
+  ((TREE_CODE (NODE) == VOID_TYPE || TREE_CODE (NODE) == RECORD_TYPE	\
+    || TREE_CODE (NODE) == UNION_TYPE)					\
    && TYPE_DUMMY_P (NODE))
 
 /* Nonzero if this corresponds to a type where alignment is guaranteed
    by other mechanisms (a tagged or packed type).  */
 #define TYPE_ALIGN_OK_P(NODE) TYPE_LANG_FLAG_5 (NODE)
 
-/* For RECORD_TYPE, nonzero if this was made for the component type
-   of an ARRAY_TYPE where we had to use max_size.  */
-#define TYPE_COMPONENT_MAX_TYPE_P(NODE) TYPE_LANG_FLAG_6 (NODE)
+/* For an INTEGER_TYPE, nonzero if TYPE_ACTUAL_BOUNDS is present.  */
+#define TYPE_HAS_ACTUAL_BOUNDS_P(NODE) TYPE_LANG_FLAG_6 (NODE)
+
+/* For a RECORD_TYPE, nonzero if this was made just to supply needed
+   padding or alignment.  */
+#define TYPE_IS_PADDING_P(NODE) TYPE_LANG_FLAG_6 (NODE)
 
 /* This field is only defined for FUNCTION_TYPE nodes. If the Ada
    subprogram contains no parameters passed by copy in/copy out then this
@@ -138,19 +164,31 @@ extern int force_global;
    the type corresponding to the Ada index type.  */
 #define TYPE_INDEX_TYPE(NODE)	(tree) TYPE_LANG_SPECIFIC (NODE)
 
+/* For INTEGER_TYPE, stores the RM_Size of the type.  */
+#define TYPE_RM_SIZE_INT(NODE)	TYPE_VALUES (NODE)
+
+/* Likewise for ENUMERAL_TYPE.  */
+#define TYPE_RM_SIZE_ENUM(NODE)	(tree) TYPE_LANG_SPECIFIC (NODE)
+
+#define TYPE_RM_SIZE(NODE)					\
+  (TREE_CODE (NODE) == ENUMERAL_TYPE ? TYPE_RM_SIZE_ENUM (NODE)	\
+   : TREE_CODE (NODE) == INTEGER_TYPE ? TYPE_RM_SIZE_INT (NODE)	\
+   : 0)
+
 /* For a RECORD_TYPE that is a fat pointer, point to the type for the
    unconstrained object.  Likewise for a RECORD_TYPE that is pointed
    to by a thin pointer.  */
 #define TYPE_UNCONSTRAINED_ARRAY(NODE)  (tree) TYPE_LANG_SPECIFIC (NODE)
 
-/* For an ARRAY_TYPE, this is the index type that should be used when
-   the actual bounds are required for a template.  This is used in
-   the case of packed arrays.  */
-#define TYPE_ACTUAL_BOUNDS(NODE)   (tree) TYPE_LANG_SPECIFIC (NODE)
+/* For other RECORD_TYPEs and all UNION_TYPEs and QUAL_UNION_TYPEs, the Ada
+   size of the object.  This differs from the GCC size in that it does not
+   include any rounding up to the alignment of the type.  */
+#define TYPE_ADA_SIZE(NODE)	(tree) TYPE_LANG_SPECIFIC (NODE)
 
-/* For a RECORD_TYPE that is not a fat pointer, point to the original subtype
-   for the _parent field, if there was one.  */
-#define TYPE_PARENT_SUBTYPE(NODE)  (tree) TYPE_LANG_SPECIFIC (NODE)
+/* For an INTEGER_TYPE with TYPE_HAS_ACTUAL_BOUNDS_P or an ARRAY_TYPE, this is
+   the index type that should be used when the actual bounds are required for
+   a template.  This is used in the case of packed arrays.  */
+#define TYPE_ACTUAL_BOUNDS(NODE)   (tree) TYPE_LANG_SPECIFIC (NODE)
 
 /* In an UNCONSTRAINED_ARRAY_TYPE, points to the record containing both
    the template and object.  */
@@ -163,26 +201,39 @@ extern int force_global;
    is needed to access the object.  */
 #define DECL_BY_REF_P(NODE) DECL_LANG_FLAG_1 (NODE)
 
-/* Nonzero in a FIELD_DECL that is a parent field.  */
-#define DECL_PARENT_P(NODE) DECL_LANG_FLAG_2 (NODE)
-
-/* Nonzero in a PARM_DECL if this was an Ada array being passed to a
+/* Nonzero if this decl is a PARM_DECL for an Ada array being passed to a
    foreign convention subprogram.  */
-#define DECL_BY_COMPONENT_PTR_P(NODE) DECL_LANG_FLAG_3 (NODE)
+#define DECL_BY_COMPONENT_PTR_P(NODE) DECL_LANG_FLAG_2 (NODE)
 
-/* Nonzero in a FIELD_DECL that is the first field in a variant of
-   record with a record rep clause.  */
-#define DECL_FIRST_FIELD_IN_VARIANT_P(NODE) DECL_LANG_FLAG_4 (NODE)
-
-/* Nonzero in a FIELD_DECL that is a dummy built for a variant part.  */
-#define DECL_FOR_VARIANT_P(NODE) DECL_LANG_FLAG_5 (NODE)
+/* Nonzero in a FIELD_DECL that is a dummy built for some internal reason.  */
+#define DECL_INTERNAL_P(NODE) DECL_LANG_FLAG_3 (NODE)
 
 /* Nonzero if this is a decl for a pointer that points to something which
    is readonly.  Used mostly for fat pointers.  */
-#define DECL_POINTS_TO_READONLY_P(NODE) DECL_LANG_FLAG_6 (NODE)
+#define DECL_POINTS_TO_READONLY_P(NODE) DECL_LANG_FLAG_4 (NODE)
+
+/* Nonzero in a FIELD_DECL if there was a record rep clause.  */
+#define DECL_HAS_REP_P(NODE) DECL_LANG_FLAG_5 (NODE)
+
+/* Nonzero in a PARM_DECL if we are to pass by descriptor.  */
+#define DECL_BY_DESCRIPTOR_P(NODE) DECL_LANG_FLAG_5 (NODE)
 
 /* Nonzero in a TRANSFORM_EXPR if we are to take its address.  */
-#define TREE_TRANSFORM_ADDR(NODE) TREE_STATIC (NODE)
+#define TREE_TRANSFORM_ADDR(NODE) TREE_LANG_FLAG_0 (NODE)
+
+/* Data structures used to represent attributes.  */
+
+enum attr_type {ATTR_MACHINE_ATTRIBUTE, ATTR_LINK_ALIAS, 
+		ATTR_LINK_SECTION, ATTR_WEAK_EXTERNAL};
+
+struct attrib
+{
+  struct attrib *next;
+  enum attr_type type;
+  tree name;
+  tree arg;
+  Node_Id error_point;
+};
 
 /* Variables expected by the GCC back-end.  */
 
@@ -238,6 +289,7 @@ extern tree get_jmpbuf_decl;
 extern tree set_jmpbuf_decl;
 extern tree get_excptr_decl;
 extern tree raise_decl;
+extern tree raise_with_msg_decl;
 extern tree raise_nodefer_decl;
 extern tree setjmp_decl;
 extern tree longjmp_decl;
@@ -366,30 +418,26 @@ extern void finish_record_type		PROTO((tree, tree, int, int));
    PARM_DECL nodes that are the subprogram arguments.  CICO_LIST is the
    copy-in/copy-out list to be stored into TYPE_CI_CO_LIST. 
    RETURNS_UNCONSTRAINED is nonzero if the function returns an unconstrained
-   object.  RETURNS_BY_REF is nonzero if the function returns by reference.  */
+   object.  RETURNS_BY_REF is nonzero if the function returns by reference. */
 extern tree create_subprog_type		PROTO((tree, tree, tree, int, int));
 
 /* Return a copy of TYPE, in the same obstack as it was, but safe to modify
    in any way.  */
-
 extern tree copy_type			PROTO((tree));
 
 /* Return an INTEGER_TYPE of SIZETYPE with range MIN to MAX and whose
    TYPE_INDEX_TYPE is INDEX.  */
 extern tree create_index_type		PROTO((tree, tree, tree));
 
-/* Returns a TYPE_DECL node. TYPE_NAME gives the name of the type (a character
-   string) and TYPE is a ..._TYPE node giving its data type.  */
-extern tree create_type_decl		PROTO((char *, tree, tree));
+/* Returns a TYPE_DECL node. TYPE_NAME gives the name of the type and TYPE
+   is a ..._TYPE node giving its data type.  */
+extern tree create_type_decl		PROTO((tree, tree, struct attrib *));
 
-/* Returns a GCC VAR_DECL node. VAR_NAME gives the name of the variable (a
-   character string). ASM_NAME is its assembler name (if provided).  TYPE is
+/* Returns a GCC VAR_DECL node. VAR_NAME gives the name of the variable.
+   ASM_NAME is its assembler name (if provided).  TYPE is
    its data type (a GCC ..._TYPE node).  VAR_INIT is the GCC tree for an
    optional initial expression; NULL_TREE if none.
 
-   SIZE, if nonzero, is a GCC tree to be used for the size of the variable.
-   ALIGN, if nonzero, is the required alignment of the variable.
-
    CONST_FLAG is nonzero if this variable is constant.
 
    PUBLIC_FLAG is nonzero if this definition is to be made visible outside of
@@ -399,8 +447,11 @@ extern tree create_type_decl		PROTO((char *, tree, tree));
    definition: no storage is to be allocated for the variable here).
    STATIC_FLAG is only relevant when not at top level.  In that case
    it indicates whether to always allocate storage to the variable.  */
-extern tree create_var_decl	PROTO((char *, char *, tree, tree, tree, int,
-				       int, int, int, int));
+extern tree create_var_decl	PROTO((tree, tree, tree, tree, int, int, int,
+				       int, struct attrib *));
+
+/* Given a DECL and ATTR_LIST, apply the listed attributes.  */
+extern void process_attributes PROTO((tree, struct attrib *));
 
 /* Obtain any pending elaborations and clear the old list.  */
 extern tree get_pending_elaborations PROTO((void));
@@ -425,31 +476,29 @@ extern void add_pending_elaborations PROTO((tree, tree));
 
 /* Returns a FIELD_DECL node. FIELD_NAME the field name, FIELD_TYPE is its
    type, and RECORD_TYPE is the type of the parent.  PACKED is nonzero if
-   this field is in a record type with a "pragma pack".  If SIZE is nonnegative
-   it is the specified size for this field and POS is the bit position.  
-   If SIZE is the special value -2, it means we should not replace a
-   discriminated size with the maximum value; this is used in laying
-   out nested variants.  */
-extern tree create_field_decl	PROTO((char *, tree, tree, int, int, int));
+   this field is in a record type with a "pragma pack".  If SIZE is nonzero
+   it is the specified size for this field.  If POS is nonzero, it is the bit
+   position.  */
+extern tree create_field_decl	PROTO((tree, tree, tree, int, tree, tree));
 
 /* Returns a PARM_DECL node. PARAM_NAME is the name of the parameter,
    PARAM_TYPE is its type.  READONLY is nonzero if the parameter is
    readonly (either an IN parameter or an address of a pass-by-ref
    parameter). */
-extern tree create_param_decl	PROTO((char *, tree, int));
+extern tree create_param_decl	PROTO((tree, tree, int));
 
 /* Returns a FUNCTION_DECL node.  SUBPROG_NAME is the name of the subprogram,
    ASM_NAME is its assembler name, SUBPROG_TYPE is its type (a FUNCTION_TYPE
    node), PARAM_DECL_LIST is the list of the subprogram arguments (a list of
    PARM_DECL nodes chained through the TREE_CHAIN field).
 
-   INLINE_FLAG, PUBLIC_FLAG, EXTERN_FLAG, and PURE_FLAG are used to set the
-   appropriate fields in the FUNCTION_DECL.  */
-extern tree create_subprog_decl	PROTO((char *, char *, tree, tree,
-				       int, int, int, int, tree));
+   INLINE_FLAG, PUBLIC_FLAG, and EXTERN_FLAG are used to set the appropriate
+   fields in the FUNCTION_DECL.  */
+extern tree create_subprog_decl	PROTO((tree, tree, tree, tree,
+				       int, int, int, struct attrib *));
 
 /* Returns a LABEL_DECL node for LABEL_NAME.  */
-extern tree create_label_decl	PROTO((char *));
+extern tree create_label_decl	PROTO((tree));
 
 /* Set up the framework for generating code for SUBPROG_DECL, a subprogram
    body. This routine needs to be invoked before processing the declarations
@@ -465,13 +514,22 @@ extern void end_subprog_body	PROTO((void));
    Return a constructor for the template.  */
 extern tree build_template	PROTO((tree, tree, tree));
 
+/* Build a VMS descriptor from a Mechanism_Type, which must specify
+   a descriptor type, and the GCC type of an object.  Each FIELD_DECL
+   in the type contains in its DECL_INITIAL the expression to use when
+   a constructor is made for the type.  GNAT_ENTITY is a gnat node used
+   to print out an error message if the mechanism cannot be applied to
+   an object of that type and also for the name.  */
+
+extern tree build_vms_descriptor  PROTO((tree, Mechanism_Type, Entity_Id));
+
 /* Build a type to be used to represent an aliased object whose nominal
    type is an unconstrained array.  This consists of a RECORD_TYPE containing
    a field of TEMPLATE_TYPE and a field of OBJECT_TYPE, which is an
    ARRAY_TYPE.  If ARRAY_TYPE is that of the unconstrained array, this
    is used to represent an arbitrary unconstrained object.  Use NAME
    as the name of the record.  */
-extern tree build_unc_object_type  PROTO((tree, tree, char *));
+extern tree build_unc_object_type  PROTO((tree, tree, tree));
 
 /* Update anything previously pointing to OLD_TYPE to point to NEW_TYPE.  In
    the normal case this is just two adjustments, but we have more to do
@@ -482,3 +540,11 @@ extern void update_pointer_to	PROTO((tree, tree));
    discriminant references, replace them with the maximum (if MAX_P) or
    minimum (if ! MAX_P) possible value of the discriminant.  */
 extern tree max_size		PROTO((tree, int));
+
+/* If EXP's type is an UNCONSTRAINED_ARRAY_TYPE, return an expression that
+   refers to the underlying array.  If its type has TYPE_CONTAINS_TEMPLATE_P,
+   likewise return an expression pointing to the underlying array.  */
+extern tree maybe_unconstrained_array PROTO((tree));
+
+/* Return an expression that does an unchecked converstion of EXPR to TYPE.  */
+extern tree unchecked_convert	PROTO((tree, tree));
