@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                          C Implementation File                           */
 /*                                                                          */
-/*                            $Revision: 1.116 $                            */
+/*                            $Revision: 1.139 $                            */
 /*                                                                          */
-/*    Copyright (C) 1992, 1993, 1994, 1995 Free Software Foundation, Inc.   */
+/*          Copyright (C) 1992-1997, Free Software Foundation, Inc.         */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -54,10 +54,12 @@ static tree nonbinary_modular_operation	PROTO((enum tree_code, tree,
 /* Prepare expr to be an argument of a TRUTH_NOT_EXPR or other logical
    operation.
 
-   This preparation consists of taking the ordinary
-   representation of an expression expr and producing a valid tree
-   boolean expression describing whether expr is nonzero.  We could
-   simply always do build_binary_op (NE_EXPR, expr, integer_zero_node, 1),
+   This preparation consists of taking the ordinary representation of
+   an expression expr and producing a valid tree boolean expression
+   describing whether expr is nonzero. We could simply always do
+
+      build_binary_op (NE_EXPR, expr, integer_zero_node, 1),
+
    but we optimize comparisons, &&, ||, and !.
 
    The resulting type should always be the same as the input type.
@@ -100,6 +102,10 @@ tree
 get_base_type (type)
      tree type;
 {
+  if (TREE_CODE (type) == RECORD_TYPE
+      && TYPE_LEFT_JUSTIFIED_MODULAR_P (type))
+    type = TREE_TYPE (TYPE_FIELDS (type));
+
   while (TREE_TYPE (type) != 0
 	 && (TREE_CODE (type) == INTEGER_TYPE
 	     || TREE_CODE (type) == REAL_TYPE))
@@ -175,7 +181,7 @@ compare_arrays (result_type, a1, a2)
   /* Process each dimension separately and compare the lengths.  If any
      dimension has a size known to be zero, set SIZE_ZERO_P to 1 to
      suppress the comparison of the data.  */
-  while (TREE_CODE (t1) == ARRAY_TYPE)
+  while (TREE_CODE (t1) == ARRAY_TYPE && TREE_CODE (t2) == ARRAY_TYPE)
     {
       tree lb1 = TYPE_MIN_VALUE (TYPE_DOMAIN (t1));
       tree ub1 = TYPE_MAX_VALUE (TYPE_DOMAIN (t1));
@@ -427,40 +433,150 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
   tree right_type = TREE_TYPE (right_operand);
   tree left_base_type = get_base_type (left_type);
   tree right_base_type = get_base_type (right_type);
-  tree operation_type = (result_type != 0
-			 && ! AGGREGATE_TYPE_P (result_type)
-			 && TYPE_EXTRA_SUBTYPE_P (result_type)
-			 ? get_base_type (result_type) : result_type);
-  tree modulus = (operation_type != 0 && TYPE_MODULAR_P (operation_type)
-		  ? TYPE_MODULUS (operation_type) : 0);
+  tree operation_type = result_type;
+  tree best_type;
+  tree modulus;
   tree result;
   int has_side_effects = 0;
 
+  if (operation_type != 0
+      && TREE_CODE (operation_type) == RECORD_TYPE
+      && TYPE_LEFT_JUSTIFIED_MODULAR_P (operation_type))
+    operation_type = TREE_TYPE (TYPE_FIELDS (operation_type));
+
+  if (operation_type != 0
+      && ! AGGREGATE_TYPE_P (operation_type)
+      && TYPE_EXTRA_SUBTYPE_P (operation_type))
+    operation_type = get_base_type (operation_type);
+
+  modulus = (operation_type != 0 && TYPE_MODULAR_P (operation_type)
+	     ? TYPE_MODULUS (operation_type) : 0);
+
   switch (op_code)
     {
     case MODIFY_EXPR:
+      /* If there were any integral or pointer conversions on LHS, remove
+	 them; we'll be putting them back below if needed.  Likewise for
+	 conversions between array and record types.  But don't do this if
+	 the right operand is not BLKmode (for packed arrays)
+	 unless we are not changing the mode.  */
+      while ((TREE_CODE (left_operand) == CONVERT_EXPR
+	      || TREE_CODE (left_operand) == NOP_EXPR
+	      || TREE_CODE (left_operand) == UNCHECKED_CONVERT_EXPR)
+	     && (((INTEGRAL_TYPE_P (left_type)
+		   || POINTER_TYPE_P (left_type))
+		  && (INTEGRAL_TYPE_P (TREE_TYPE
+				       (TREE_OPERAND (left_operand, 0)))
+		      || POINTER_TYPE_P (TREE_TYPE
+					 (TREE_OPERAND (left_operand, 0)))))
+		 || (((TREE_CODE (left_type) == RECORD_TYPE
+		       /* Don't remove conversions to left-justified modular
+			  types. */
+		       && ! TYPE_LEFT_JUSTIFIED_MODULAR_P (left_type))
+		      || TREE_CODE (left_type) == ARRAY_TYPE)
+		     && ((TREE_CODE (TREE_TYPE
+				     (TREE_OPERAND (left_operand, 0)))
+			  == RECORD_TYPE)
+			 || (TREE_CODE (TREE_TYPE
+					(TREE_OPERAND (left_operand, 0)))
+			     == ARRAY_TYPE))
+		     && (TYPE_MODE (right_type) == BLKmode
+			 || (TYPE_MODE (left_type)
+			     == TYPE_MODE (TREE_TYPE
+					   (TREE_OPERAND
+					    (left_operand, 0))))))))
+	{
+	  left_operand = TREE_OPERAND (left_operand, 0);
+	  left_type = TREE_TYPE (left_operand);
+	}
+
       if (operation_type == 0)
 	operation_type = left_type;
 
+      /* If the RHS has a conversion between record and array types and
+	 an inner type is no worse, use it.  Note we cannot do this for
+	 modular types or types with TYPE_ALIGN_OK_P, since the latter
+	 might indicate a conversion between a root type and a class-wide
+	 type, which we must not remove.  */
+      while (TREE_CODE (right_operand) == UNCHECKED_CONVERT_EXPR
+	     && ((TREE_CODE (right_type) == RECORD_TYPE
+		  && ! TYPE_LEFT_JUSTIFIED_MODULAR_P (right_type)
+		  && ! TYPE_ALIGN_OK_P (right_type))
+		 || TREE_CODE (right_type) == ARRAY_TYPE)
+	     && ((TREE_CODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
+		  == RECORD_TYPE)
+		 || (TREE_CODE (TREE_TYPE (TREE_OPERAND (right_operand, 0)))
+		     == ARRAY_TYPE))
+	     && (right_type
+		 != find_common_type (right_type,
+				      TREE_TYPE (TREE_OPERAND
+						 (right_operand, 0)))))
+	{
+	  right_operand = TREE_OPERAND (right_operand, 0);
+	  right_type = TREE_TYPE (right_operand);
+	}
+
       /* If we are copying one array or record to another, find the best type
 	 to use.  */
-      if ((TREE_CODE (left_type) == ARRAY_TYPE
-	   && TREE_CODE (right_type) == ARRAY_TYPE)
-	  || (TREE_CODE (left_type) == RECORD_TYPE
-	      && TREE_CODE (right_type) == RECORD_TYPE))
+      if (((TREE_CODE (left_type) == ARRAY_TYPE
+	    && TREE_CODE (right_type) == ARRAY_TYPE)
+	   || (TREE_CODE (left_type) == RECORD_TYPE
+	       && TREE_CODE (right_type) == RECORD_TYPE))
+	  && (best_type = find_common_type (left_type, right_type)) != 0)
+	operation_type = best_type;
+
+      /* If a class-wide type may be involved, force use of the RHS type.  */
+      if (TREE_CODE (right_type) == RECORD_TYPE
+	  && TYPE_ALIGN_OK_P (right_type))
+	operation_type = right_type;
+
+      /* After we strip off any COMPONENT_REF or ARRAY_REF from the lhs, we
+	 must have either an INDIRECT_REF or a decl. Allow
+	 UNCHECKED_CONVERT_EXPRs, but set TREE_ADDRESSABLE to show they are
+	 in an LHS.  Finally, allow NOP_EXPR if both types are the same tree
+	 code and mode because we know these will be nops.  */
+      for (result = left_operand;
+	   TREE_CODE (result) == COMPONENT_REF
+	   || TREE_CODE (result) == ARRAY_REF
+	   || TREE_CODE (result) == REALPART_EXPR
+	   || TREE_CODE (result) == IMAGPART_EXPR
+	   || TREE_CODE (result) == UNCHECKED_CONVERT_EXPR
+	   || (TREE_CODE (result) == NOP_EXPR
+	       && (TREE_CODE (TREE_TYPE (result))
+		   == TREE_CODE (TREE_TYPE (TREE_OPERAND (result, 0))))
+	       && (TYPE_MODE (TREE_TYPE (TREE_OPERAND (result, 0)))
+		   == TYPE_MODE (TREE_TYPE (result))));
+	   result = TREE_OPERAND (result, 0))
+	if (TREE_CODE (result) == UNCHECKED_CONVERT_EXPR)
+	  TREE_ADDRESSABLE (result) = 1;
+
+      if (TREE_CODE (result) != INDIRECT_REF
+	  && TREE_CODE (result) != NULL_EXPR
+	  && TREE_CODE_CLASS (TREE_CODE (result)) != 'd')
+	gigi_abort (516);
+
+      /* Convert the right operand to the operation type unless
+	 it is either already of the correct type or if the type
+	 involves a placeholder, since the RHS may not have the same
+	 record type.  */
+      if (operation_type != right_type
+	  && (! (TREE_CODE (TYPE_SIZE (operation_type)) != INTEGER_CST
+		 && contains_placeholder_p (TYPE_SIZE (operation_type)))))
 	{
-	  tree best_type = find_common_type (left_type, right_type);
+	  right_operand = convert (operation_type, right_operand);
+	  right_type = operation_type;
+	}
 
-	  if (best_type && left_type != best_type)
-	    left_operand = convert (best_type, left_operand);
-	  if (best_type && right_type != best_type)
-	    right_operand = convert (best_type, right_operand);
+      /* If the modes differ, make up a bogus type and convert the RHS to
+	 it.  This can happen with packed types.  */
+      if (TYPE_MODE (left_type) != TYPE_MODE (right_type))
+	{
+	  tree new_type = copy_node (left_type);
 
-	  if (best_type)
-	    operation_type = best_type;
+	  TYPE_SIZE (new_type) = TYPE_SIZE (right_type);
+	  TYPE_MAIN_VARIANT (new_type) = new_type;
+	  right_operand = convert (new_type, right_operand);
 	}
-      else
-	right_operand = convert (operation_type, right_operand);
 
       has_side_effects = 1;
       modulus = 0;
@@ -494,9 +610,41 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 
     case EQ_EXPR:
     case NE_EXPR:
+      /* If either operand is a NULL_EXPR, just return a new one.  */
+      if (TREE_CODE (left_operand) == NULL_EXPR
+	  || TREE_CODE (right_operand) == NULL_EXPR)
+	return build (op_code, result_type,
+		      build1 (NULL_EXPR, integer_type_node,
+			      TREE_OPERAND (left_operand, 0)),
+		      integer_zero_node);
+
+      /* If either object is a left-justified modular types, get the
+	 fields from within.  */
+      if (TREE_CODE (left_type) == RECORD_TYPE
+	  && TYPE_LEFT_JUSTIFIED_MODULAR_P (left_type))
+	{
+	  left_operand = convert (TREE_TYPE (TYPE_FIELDS (left_type)),
+				  left_operand);
+	  left_type = TREE_TYPE (left_operand);
+	  left_base_type = get_base_type (left_type);
+	}
+
+      if (TREE_CODE (right_type) == RECORD_TYPE
+	  && TYPE_LEFT_JUSTIFIED_MODULAR_P (right_type))
+	{
+	  right_operand = convert (TREE_TYPE (TYPE_FIELDS (right_type)),
+				  right_operand);
+	  right_type = TREE_TYPE (right_operand);
+	  right_base_type = get_base_type (right_type);
+	}
+
       /* If both objects are arrays, compare them specially.  */
-      if (TREE_CODE (left_type) == ARRAY_TYPE
-	  && TREE_CODE (right_type) == ARRAY_TYPE)
+      if ((TREE_CODE (left_type) == ARRAY_TYPE
+	   || (TREE_CODE (left_type) == INTEGER_TYPE
+	       && TYPE_HAS_ACTUAL_BOUNDS_P (left_type)))
+	  && (TREE_CODE (right_type) == ARRAY_TYPE
+	      || (TREE_CODE (right_type) == INTEGER_TYPE
+		  && TYPE_HAS_ACTUAL_BOUNDS_P (right_type))))
 	{
 	  result = compare_arrays (result_type, left_operand, right_operand);
 
@@ -522,7 +670,7 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 		 types have the same name.  In that case, one of them must
 		 not be self-referential.  Use that one as the best type.
 		 Even better is if one is of fixed size.  */
-	      tree best_type = 0;
+	      best_type = 0;
 
 	      if (TYPE_NAME (left_base_type) == 0
 		  || TYPE_NAME (left_base_type) != TYPE_NAME (right_base_type))
@@ -547,15 +695,13 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 	}
 
       /* If we are comparing a fat pointer against zero, we need to 
-	 just compare the template pointer.  */
+	 just compare the data pointer.  */
       else if (TYPE_FAT_POINTER_P (left_base_type)
 	       && TREE_CODE (right_operand) == CONSTRUCTOR
 	       && integer_zerop (TREE_VALUE (TREE_OPERAND (right_operand, 1))))
 	{
-	  right_operand
-	    = build_component_ref (left_operand,
-				   NULL_TREE,
-				   TREE_CHAIN (TYPE_FIELDS (left_base_type)));
+	  right_operand = build_component_ref (left_operand, NULL_TREE,
+					       TYPE_FIELDS (left_base_type));
 	  left_operand = convert (TREE_TYPE (right_operand),
 				  integer_zero_node);
 	}
@@ -594,12 +740,13 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
     case RSHIFT_EXPR:
     case LROTATE_EXPR:
     case RROTATE_EXPR:
-      /* The RHS of a shift can be any type.  In addition, we don't support
-	 them on modular types.  Otherwise, processing is the same as
-	 normal.  */
-      if (operation_type != left_base_type || modulus != 0)
+      /* The RHS of a shift can be any type.  Also, ignore any modulus
+	 (we used to abort, but this is needed for unchecked conversion
+	 to modular types).  Otherwise, processing is the same as normal.  */
+      if (operation_type != left_base_type)
 	gigi_abort (514);
 
+      modulus = 0;
       left_operand = convert (operation_type, left_operand);
       break;
 
@@ -622,6 +769,12 @@ build_binary_op (op_code, result_type, left_operand, right_operand)
 
       goto common;
 
+    case COMPLEX_EXPR:
+      if (TREE_TYPE (result_type) != left_type
+	  || TREE_TYPE (result_type) != right_type)
+	gigi_abort (515);
+      break;
+
     case TRUNC_DIV_EXPR:   case TRUNC_MOD_EXPR:
     case CEIL_DIV_EXPR:    case CEIL_MOD_EXPR:
     case FLOOR_DIV_EXPR:   case FLOOR_MOD_EXPR:
@@ -681,15 +834,32 @@ build_unary_op (op_code, result_type, operand)
 {
   tree type = TREE_TYPE (operand);
   tree base_type = get_base_type (type);
-  tree operation_type = (result_type != 0
-			 && ! AGGREGATE_TYPE_P (result_type)
-			 && TYPE_EXTRA_SUBTYPE_P (result_type)
-			 ? get_base_type (result_type) : result_type);
+  tree operation_type = result_type;
   tree result;
   int side_effects = 0;
 
+  if (operation_type != 0
+      && TREE_CODE (operation_type) == RECORD_TYPE
+      && TYPE_LEFT_JUSTIFIED_MODULAR_P (operation_type))
+    operation_type = TREE_TYPE (TYPE_FIELDS (operation_type));
+
+  if (operation_type != 0
+      && ! AGGREGATE_TYPE_P (operation_type)
+      && TYPE_EXTRA_SUBTYPE_P (operation_type))
+    operation_type = get_base_type (operation_type);
+
   switch (op_code)
     {
+    case REALPART_EXPR:
+    case IMAGPART_EXPR:
+      if (operation_type == 0)
+	result_type = operation_type = TREE_TYPE (type);
+      else if (result_type != TREE_TYPE (type))
+	gigi_abort (513);
+
+      result = fold (build1 (op_code, operation_type, operand));
+      break;
+
     case TRUTH_NOT_EXPR:
       if (result_type != base_type)
 	gigi_abort (508);
@@ -698,11 +868,19 @@ build_unary_op (op_code, result_type, operand)
       break;
 
     case ADDR_EXPR:
-      if (TREE_CODE (operand) == INDIRECT_REF
-	  || TREE_CODE (operand) == UNCONSTRAINED_ARRAY_REF)
-	result = TREE_OPERAND (operand, 0);
-      else if (TREE_CODE (operand) == TRANSFORM_EXPR)
+      switch (TREE_CODE (operand))
 	{
+	case INDIRECT_REF:
+	case UNCONSTRAINED_ARRAY_REF:
+	  result = TREE_OPERAND (operand, 0);
+	  break;
+
+	case NULL_EXPR:
+	  result = operand;
+	  TREE_TYPE (result) = type = build_pointer_type (type);
+	  break;
+
+	case TRANSFORM_EXPR:
 	  TREE_TRANSFORM_ADDR (operand) = 1;
 	  result = operand;
 
@@ -711,40 +889,44 @@ build_unary_op (op_code, result_type, operand)
 	      type = build_pointer_type (type);
 	      TREE_TYPE (result) = type;
 	    }
-	}
-      else if (TREE_CODE (operand) == ARRAY_REF)
-	{
-	  /* Compute address of array element by adding address of array
-	     to index times size of element.  */
-	  tree array = TREE_OPERAND (operand, 0);
-	  tree array_adr = build1 (ADDR_EXPR,
-				   build_pointer_type (type), array);
-	  tree low_bound = TYPE_MIN_VALUE (TYPE_DOMAIN (TREE_TYPE (array)));
-	  tree index = TREE_OPERAND (operand, 1);
-	  tree size = size_in_bytes (type);
+	  break;
 
-	  mark_addressable (array);
+	case ARRAY_REF:
+	  {
+	    /* Compute address of array element by adding address of array
+	       to index times size of element.  */
+	    tree array = TREE_OPERAND (operand, 0);
+	    tree array_adr = build1 (ADDR_EXPR,
+				     build_pointer_type (type), array);
+	    tree low_bound = TYPE_MIN_VALUE (TYPE_DOMAIN (TREE_TYPE (array)));
+	    tree index = TREE_OPERAND (operand, 1);
+	    tree size = size_in_bytes (type);
 
-	  if (TREE_CODE (low_bound) != INTEGER_CST
-	      && contains_placeholder_p (low_bound))
-	    low_bound = build (WITH_RECORD_EXPR, sizetype, low_bound, operand);
+	    mark_addressable (array);
 
-	  /* Convert the integer argument to a type the same size as a
-	     pointer so the arithmetic won't overflow spuriously.  */
-	  if (TYPE_PRECISION (TREE_TYPE (index)) != POINTER_SIZE)
-	    index = convert (type_for_size (POINTER_SIZE, 0), index);
+	    if (TREE_CODE (low_bound) != INTEGER_CST
+		&& contains_placeholder_p (low_bound))
+	      low_bound = build (WITH_RECORD_EXPR, sizetype,
+				 low_bound, operand);
 
-	  index = fold (build (MINUS_EXPR, TREE_TYPE (index), index,
-			       convert (TREE_TYPE (index), low_bound)));
+	    /* Convert the integer argument to a type the same size as a
+	       pointer so the arithmetic won't overflow spuriously.  */
+	    if (TYPE_PRECISION (TREE_TYPE (index)) != POINTER_SIZE)
+	      index = convert (type_for_size (POINTER_SIZE, 0), index);
 
-	  index = fold (build (MULT_EXPR, TREE_TYPE (index),
-			       index, convert (TREE_TYPE (index), size)));
+	    index = fold (build (MINUS_EXPR, TREE_TYPE (index), index,
+				 convert (TREE_TYPE (index), low_bound)));
+
+	    index = fold (build (MULT_EXPR, TREE_TYPE (index),
+				 index, convert (TREE_TYPE (index), size)));
+
+	    result = fold (build (PLUS_EXPR, TYPE_POINTER_TO (type), array_adr,
+				  convert (TYPE_POINTER_TO (type), index)));
+	  }
+	  break;
+
+	default:
 
-	  result = fold (build (PLUS_EXPR, TYPE_POINTER_TO (type), array_adr,
-				convert (TYPE_POINTER_TO (type), index)));
-	}
-      else
-	{
 	  if (type != error_mark_node)
 	    operation_type = build_pointer_type (type);
 
@@ -760,8 +942,7 @@ build_unary_op (op_code, result_type, operand)
 	 make up an expression to do so.  This will never survive to
 	 the backend.  If TYPE is a thin pointer, first convert the
 	 operand to a fat pointer.  */
-      if (TREE_CODE (type) == POINTER_TYPE
-	  && TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (type))
+      if (TYPE_THIN_POINTER_P (type)
 	  && TYPE_UNCONSTRAINED_ARRAY (TREE_TYPE (type)) != 0)
 	{
 	  operand
@@ -947,7 +1128,10 @@ build_constructor (type, list)
 
   for (elmt = list; elmt; elmt = TREE_CHAIN (elmt))
     {
-      if (! TREE_CONSTANT (TREE_VALUE (elmt)))
+      if (! TREE_CONSTANT (TREE_VALUE (elmt))
+	  || (TREE_CODE (type) == RECORD_TYPE
+	      && DECL_BIT_FIELD (TREE_PURPOSE (elmt))
+	      && TREE_CODE (TREE_VALUE (elmt)) != INTEGER_CST))
 	allconstant = 0;
 
       if (TREE_SIDE_EFFECTS (TREE_VALUE (elmt)))
@@ -997,7 +1181,7 @@ build_simple_component_ref (record_variable, component, field)
      tree component;
      tree field;
 {
-  tree record_type = TREE_TYPE (record_variable);
+  tree record_type = TYPE_MAIN_VARIANT (TREE_TYPE (record_variable));
   tree ref;
 
   /* Handle added pointer for pass-by-reference values.  */
@@ -1028,7 +1212,7 @@ build_simple_component_ref (record_variable, component, field)
 	{
 	  if (DECL_NAME (field) == component)
 	    break;
-	  else if (DECL_FOR_VARIANT_P (field))
+	  else if (DECL_INTERNAL_P (field))
 	     {
 	      tree field_ref
 		= build_simple_component_ref (record_variable, 
@@ -1045,6 +1229,8 @@ build_simple_component_ref (record_variable, component, field)
   if (!field)
     return 0;
 
+  /* It would be nice to call "fold" here, but that can lose a type
+     we need to tag a PLACEHOLDER_EXPR with, so we can't do it.  */
   ref = build (COMPONENT_REF, TREE_TYPE (field), record_variable, field);
 
   if (TREE_READONLY (record_variable) || TREE_READONLY (field))
@@ -1055,8 +1241,8 @@ build_simple_component_ref (record_variable, component, field)
   return ref;
 }
 
-/* Like build_simple_component_ref, except that we look in the field __parent
-   if the field is not defined at this level */
+/* Like build_simple_component_ref, except that we give an error if the
+   reference could not be found.  */
 
 tree
 build_component_ref (record_variable, component, field)
@@ -1064,32 +1250,19 @@ build_component_ref (record_variable, component, field)
      tree component;
      tree field;
 {
-  tree local_field;
-  tree parent_ref;
-  static tree parent_comp = 0;
-  tree comp_ref;
+  tree ref = build_simple_component_ref (record_variable, component, field);
 
-  if (!parent_comp) 
-    parent_comp = get_identifier (Get_Name_String (Name_uParent)); 
-
-  /* See if the field is present at this level.  */
-  comp_ref = build_simple_component_ref (record_variable, component, field);
-  if (comp_ref)
-    return comp_ref;
-
-  /* If it is not present, look recursively in the parent.  */
-
-  parent_ref = build_simple_component_ref (record_variable, 
-					   parent_comp, NULL_TREE);
-  if (parent_ref)
-    return build_component_ref (parent_ref, component, field);
+  if (ref != 0)
+    return ref;
 
   /* If FIELD was specified, assume this is an invalid user field so
      raise constraint error.  Otherwise, we can't find the type to return, so
      abort.  */
+
   else if (field != 0)
     return build1 (NULL_EXPR, TREE_TYPE (field),
 		   build_call_0_expr (raise_constraint_error_decl));
+
   else
     gigi_abort (512);
 }
@@ -1223,14 +1396,15 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   /* If RESULT_TYPE is a fat or thin pointer, set SIZE to be the sum of the
      sizes of the object and its template.  Allocate the whole thing and
      fill in the parts that are known.  */
-  if (TYPE_FAT_POINTER_P (result_type)
-      || (TYPE_CONTAINS_TEMPLATE_P (TREE_TYPE (result_type))))
+  if (TYPE_FAT_OR_THIN_POINTER_P (result_type))
     {
       tree template_type
 	= (TYPE_FAT_POINTER_P (result_type)
 	   ? TREE_TYPE (TREE_TYPE (TREE_CHAIN (TYPE_FIELDS (result_type))))
 	   : TREE_TYPE (TYPE_FIELDS (TREE_TYPE (result_type))));
-      tree storage_type = build_unc_object_type (template_type, type, "ALLOC");
+      tree storage_type
+	= build_unc_object_type (template_type, type,
+				 get_identifier ("ALLOC"));
       tree storage_ptr_type = build_pointer_type (storage_type);
       tree storage;
       tree template_cons = NULL_TREE;
@@ -1246,22 +1420,48 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
 					  gnat_proc, gnat_pool);
       storage = convert (storage_ptr_type, storage);
 
-      if (init != 0)
-	template_cons = tree_cons (TREE_CHAIN (TYPE_FIELDS (storage_type)),
-				   init, NULL_TREE);
-      template_cons = tree_cons (TYPE_FIELDS (storage_type),
-				 build_template (template_type, type, init),
-				 template_cons);
+      if (TREE_CODE (type) == RECORD_TYPE && TYPE_IS_PADDING_P (type))
+	{
+	  type = TREE_TYPE (TYPE_FIELDS (type));
 
-      return convert
-	(result_type,
-	 build_unary_op
-	 (ADDR_EXPR, NULL_TREE,
-	  build_binary_op
-	  (MODIFY_EXPR, storage_type,
-	   build_unary_op (INDIRECT_REF, NULL_TREE,
-			   convert (storage_ptr_type, storage)),
-	   build_constructor (storage_type, template_cons))));
+	  if (init != 0)
+	    init = convert (type, init);
+	}
+
+      /* If there is an initializing expression, make a constructor for
+	 the entire object including the bounds and copy it into the
+	 object.  If there is no initializing expression, just set the
+	 bounds.  */
+      if (init != 0)
+	{
+	  template_cons = tree_cons (TREE_CHAIN (TYPE_FIELDS (storage_type)),
+				     init, NULL_TREE);
+	  template_cons = tree_cons (TYPE_FIELDS (storage_type),
+				     build_template (template_type, type,
+						     init),
+				     template_cons);
+
+	  return convert
+	    (result_type,
+	     build_unary_op
+	     (ADDR_EXPR, NULL_TREE,
+	      build_binary_op
+	      (MODIFY_EXPR, storage_type,
+	       build_unary_op (INDIRECT_REF, NULL_TREE,
+			       convert (storage_ptr_type, storage)),
+	       build_constructor (storage_type, template_cons))));
+	}
+      else
+	return build
+	  (COMPOUND_EXPR, result_type,
+	   build_binary_op
+	   (MODIFY_EXPR, template_type,
+	    build_component_ref
+	    (build_unary_op (INDIRECT_REF, NULL_TREE,
+			     convert (storage_ptr_type, storage)),
+	     NULL_TREE, TYPE_FIELDS (storage_type)),
+	    build_template (template_type, type, NULL_TREE)),
+	   convert (result_type, convert (storage_ptr_type, storage)));
     }
 
   /* If we have an initializing expression, see if its size is simpler
@@ -1288,13 +1488,16 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
      allocate an object of TYPE and point to it.
      ??? At some point, we should make an attempt to do this above.  */
   if (global_bindings_p () && TREE_CODE (size) == INTEGER_CST
-      && ! TREE_OVERFLOW (size) && 0)
+      && ! TREE_OVERFLOW (size) && TREE_INT_CST_HIGH (size) == 0
+      && TREE_INT_CST_LOW (size) < 200000
+      && 0)
     {
       char name[20];
 
+      type = maybe_pad_type (type, size, 0, Empty, NULL_PTR, 0);
       sprintf (name, "__V%d", alloc_var_index++);
-      result
-	= create_var_decl (name, NULL_PTR, type, init, size, 0, 0, 0, 0, 1);
+      result = create_var_decl (get_identifier (name), NULL_TREE, type, init,
+				0, 0, 0, 1, NULL_PTR);
       result = build_unary_op (ADDR_EXPR, NULL_TREE, result);
       init = 0;
     }
@@ -1317,16 +1520,47 @@ build_allocator (type, init, result_type, gnat_proc, gnat_pool)
   if (init)
     {
       result = save_expr (result);
-      result = build (COMPOUND_EXPR, TREE_TYPE (result),
-		      build_binary_op (MODIFY_EXPR, type,
-				       build_unary_op (INDIRECT_REF, type,
-						       result),
-				       init),
-		      result);
+      result
+	= build (COMPOUND_EXPR, TREE_TYPE (result),
+		 build_binary_op
+		 (MODIFY_EXPR, TREE_TYPE (TREE_TYPE (result)),
+		  build_unary_op (INDIRECT_REF, TREE_TYPE (TREE_TYPE (result)),
+				  result),
+		  init),
+		 result);
     }
 
   return convert (result_type, result);
 }
+
+/* Fill in a VMS descriptor for EXPR and return a constructor for it. 
+   GNAT_FORMAL is how we find the descriptor record.  */
+
+tree
+fill_vms_descriptor (expr, gnat_formal)
+     tree expr;
+     Entity_Id gnat_formal;
+{
+  tree record_type = TREE_TYPE (TREE_TYPE (get_gnu_tree (gnat_formal)));
+  tree field;
+  tree const_list = 0;
+
+  expr = maybe_unconstrained_array (expr);
+
+  for (field = TYPE_FIELDS (record_type); field; field = TREE_CHAIN (field))
+    {
+      tree init = DECL_INITIAL (field);
+
+      if (TREE_CODE (init) != INTEGER_CST
+	  && contains_placeholder_p (init))
+	init = build (WITH_RECORD_EXPR, TREE_TYPE (init), init, expr);
+
+      const_list = tree_cons (field, convert (TREE_TYPE (field), init),
+			      const_list);
+    }
+
+  return build_constructor (record_type, nreverse (const_list));
+}
 
 /* Indicate that we need to make the address of EXPR_NODE and it therefore
    should not be allocated in a register. Return 1 if successful.  */
