@@ -6,9 +6,9 @@
 /*                                                                          */
 /*                             Utility Program                              */
 /*                                                                          */
-/*                            $Revision: 1.10 $                             */
+/*                            $Revision: 1.17 $                             */
 /*                                                                          */
-/*           Copyright (c) 1992,1993,1994 NYU, All Rights Reserved          */
+/*   Copyright (C) 1992,1993,1994,1995,1996 Free Software Foundation, Inc.  */
 /*                                                                          */
 /* GNAT is free software;  you can  redistribute it  and/or modify it under */
 /* terms of the  GNU General Public License as published  by the Free Soft- */
@@ -18,7 +18,11 @@
 /* or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License */
 /* for  more details.  You should have  received  a copy of the GNU General */
 /* Public License  distributed with GNAT;  see file COPYING.  If not, write */
-/* to the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA. */
+/* to  the Free Software Foundation,  59 Temple Place - Suite 330,  Boston, */
+/* MA 02111-1307, USA.                                                      */
+/*                                                                          */
+/* GNAT was originally developed  by the GNAT team at  New York University. */
+/* It is now maintained by Ada Core Technologies Inc (http://www.gnat.com). */
 /*                                                                          */
 /****************************************************************************/
 
@@ -46,145 +50,176 @@
 
 /* Map attribute in template into string value.  */
 
-#define NUM_FLOAT_TYPES (4)
-struct {char *attr, *(string[NUM_FLOAT_TYPES]);} attribute_map[] =
+#define NUM_FLOAT_SIZES 5
+
+int float_sizes[NUM_FLOAT_SIZES] = {32, 48, 64, 96, 128};
+
+struct {char *attr, *string[NUM_FLOAT_SIZES];} attribute_map[] =
 
     {{"DN",                                           /* Denorm  */
        {"True",
+        "True",
         "True",
         "True",
         "True"}},
 
      {"EP",                                           /* Epsilon  */
-       {"2#1.0#E-23",
-        "2#1.0#E-52",
-        "2#1.0#E-63",
-        "2#1.0#E-113"}},
-
+       {"2#1.0#E-20",
+        "2#1.0#E-39",
+        "2#1.0#E-49",
+        "2#1.0#E-60",
+        "2#1.0#E-60"}},
 
      {"EX",                                           /* Emax  */
-       {"128",
-        "1024",
-        "16384",
-        "16384"}},
+       {"84",
+        "156",
+        "200",
+        "244",
+        "244"}},
 
      {"FS",                                           /* First  */
        {"-16#0.FFFF_FF#E+32",
+        "-16#0.FFFF_FFFF_FF#E+64",
         "-16#0.FFFF_FFFF_FFFF_F8#E+256",
         "-16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
         "-16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
 
-
      {"LG",                                           /* Large  */
-       {"16#0.FFFF_FF#E+32",
-        "16#0.FFFF_FFFF_FFFF_F8#E+256",
-        "16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
-        "16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
+       {"16#0.FFFF_F8#E+21",
+        "16#0.FFFF_FFFF_F8#E+37",
+        "16#0.FFFF_FFFF_FFFF_C#E+50",
+        "16#0.FFFF_FFFF_FFFF_FFF8#E+61",
+        "16#0.FFFF_FFFF_FFFF_FFF8#E+61"}},
 
      {"LS",                                           /* Last  */
        {"16#0.FFFF_FF#E+32",
+        "16#0.FFFF_FFFF_FF#E+64",
         "16#0.FFFF_FFFF_FFFF_F8#E+256",
         "16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
         "16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
 
      {"MA",                                           /* Mantissa  */
-       {"24",
-        "53",
-        "64",
-        "113"}},
+       {"21",
+        "35",
+        "51",
+        "61",
+        "61"}},
 
      {"MM",                                           /* Machine_Mantissa  */
        {"24",
+        "40",
         "53",
         "64",
         "113"}},
 
      {"MN",                                           /* Machine_Emin  */
        {"-125",
+        "-125",
         "-1021",
         "-16381",
         "-16381"}},
 
      {"MO",                                           /* Machine_Overflows  */
        {"False",
+        "False",
         "False",
         "False",
         "False"}},
 
      {"MR",                                           /* Machine_Radix  */
        {"2",
+        "2",
         "2",
         "2",
         "2"}},
 
      {"MX",                                           /* Machine_Emax  */
        {"128",
+        "128",
         "1024",
         "16384",
         "16384"}},
 
      {"OE",                                           /* Model_Epsilon  */
        {"2#1.0#E-23",
+        "2#1.0#E-39",
         "2#1.0#E-52",
         "2#1.0#E-63",
         "2#1.0#E-113"}},
 
      {"OM",                                           /* Model_Mantissa  */
        {"24",
+        "40",
         "53",
         "64",
         "113"}},
 
      {"ON",                                           /* Model_Emin  */
        {"-125",
+        "-125",
         "-1021",
         "-16381",
         "-16381"}},
 
      {"OS",                                           /* Model_Small  */
        {"2#1.0#E-126",
+        "2#1.0#E-126",
         "2#1.0#E-1022",
         "2#1.0#E-16381",
         "2#1.0#E-16381"}},
 
      {"RN",                                           /* Machine_Rounds  */
        {"True",
+        "True",
         "True",
         "True",
         "True"}},
 
+     {"SA",                                           /* Small */
+       {"2#1.0#E-85",
+        "2#1.0#E-157",
+        "2#1.0#E-201",
+        "2#1.0#E-245",
+        "2#1.0#E-245"}},
+
      {"SF",                                           /* Safe_First  */
        {"-16#0.FFFF_FF#E+32",
+        "-16#0.FFFF_FFFF_FF#E+64",
         "-16#0.FFFF_FFFF_FFFF_F8#E+256",
         "-16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
         "-16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
 
      {"SL",                                           /* Safe_Large  */
        {"16#0.FFFF_FF#E+32",
+        "16#0.FFFF_FFFF_FF#E+64",
         "16#0.FFFF_FFFF_FFFF_F8#E+256",
         "16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
         "16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
 
      {"SM",                                           /* Safe_Small  */
        {"2#1.0#E-126",
+        "2#1.0#E-126",
         "2#1.0#E-1022",
         "2#1.0#E-16381",
         "2#1.0#E-16381"}},
 
      {"SS",                                           /* Safe_Last  */
        {"16#0.FFFF_FF#E+32",
+        "16#0.FFFF_FFFF_FF#E+64",
         "16#0.FFFF_FFFF_FFFF_F8#E+256",
         "16#0.FFFF_FFFF_FFFF_FFFF#E+4096",
         "16#0.FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_FFFF_8#E+4096"}},
 
      {"SX",                                           /* Safe_Emax  */
        {"128",
+        "128",
         "1024",
         "16384",
         "16384"}},
 
      {"SZ",                                           /* Signed_Zeros  */
        {"True",
+        "True",
         "True",
         "True",
         "True"}}};
@@ -192,12 +227,18 @@ struct {char *attr, *(string[NUM_FLOAT_TYPES]);} attribute_map[] =
 #define NUM_ATTRS sizeof attribute_map / sizeof attribute_map[0]
 #define UNKNOWN_FLOAT_TYPE (-1)
 
+struct {char *name; int size, idx;} float_type_sizes[] =
+{ {"float", FLOAT_TYPE_SIZE, -1},
+  {"double float", DOUBLE_TYPE_SIZE, -1},
+  {"long double", LONG_DOUBLE_TYPE_SIZE, -1}};
+
 int
 main (argc, argv)
      int argc;
      char **argv;
 {
   FILE *infile;
+  int i, j;
   int c, exit_code = SUCCESS_EXIT_CODE;
 
   if (TARGET_FLOAT_FORMAT != IEEE_FLOAT_FORMAT)
@@ -206,29 +247,19 @@ main (argc, argv)
       exit_code = FATAL_EXIT_CODE;
     }
 
-  if (FLOAT_TYPE_SIZE != 32)
+  for (i = 0; i < sizeof float_type_sizes / sizeof float_type_sizes[0]; i++)
     {
-      fprintf (stderr,
- "The characteristics of a float type of size %d bits are unknown",
-	  FLOAT_TYPE_SIZE);
-      exit_code = FATAL_EXIT_CODE;
-    }
+      for (j = 0; j < NUM_FLOAT_SIZES; j++)
+	if (float_sizes[j] == float_type_sizes[i].size)
+	  float_type_sizes[i].idx = j;
 
-  if (DOUBLE_TYPE_SIZE != 64)
-    {
-      fprintf (stderr,
- "The characteristics of a double float type of size %d bits are unknown",
-	  DOUBLE_TYPE_SIZE);
-      exit_code = FATAL_EXIT_CODE;
-    }
-
-  if (LONG_DOUBLE_TYPE_SIZE != 64 && LONG_DOUBLE_TYPE_SIZE != 96
-      && LONG_DOUBLE_TYPE_SIZE != 128)
-    {
-      fprintf (stderr,
- "The characteristics of a long double float type of size %d bits are unknown",
-	  LONG_DOUBLE_TYPE_SIZE);
-      exit_code = FATAL_EXIT_CODE;
+      if (float_type_sizes[i].idx < 0)
+	{
+	  fprintf (stderr,
+		   "Characteristics of a %s type of size %d bits unknown.",
+		   float_type_sizes[i].name, float_type_sizes[i].size);
+	  exit_code = FATAL_EXIT_CODE;
+	}
     }
 
   if (exit_code != SUCCESS_EXIT_CODE)
@@ -252,7 +283,7 @@ main (argc, argv)
 	putchar (c);
       else
 	{
-	  int float_type, c1, c2, i;
+	  int float_type, c1, c2;
 	  char *value;
 
 	  /* Get the prefix.  */
@@ -270,10 +301,7 @@ main (argc, argv)
 
 	    case 'D':
 	      /* LONG_LONG_FLOAT  */
-	      float_type = LONG_DOUBLE_TYPE_SIZE == 64 ? 1
-			     : LONG_DOUBLE_TYPE_SIZE == 96 ? 2
-			     : LONG_DOUBLE_TYPE_SIZE == 128 ? 3
-			     : UNKNOWN_FLOAT_TYPE;
+	      float_type = 2;
 	      break;
 
 	    default:
@@ -287,8 +315,8 @@ main (argc, argv)
 	  for (i = 0; i < NUM_ATTRS; i++)
 	    if (attribute_map[i].attr[0] == c1
 		&& attribute_map[i].attr[1] == c2)
-	      if (float_type >= 0 && float_type < NUM_FLOAT_TYPES)
-		value = attribute_map[i].string[float_type];
+	      value
+		= attribute_map[i].string[float_type_sizes[float_type].idx];
 
 	  if (value == NULL)
 	    {
